package com.caimei365.wechat.controller;

import com.caimei365.wechat.entity.WeChatConstant;
import com.caimei365.wechat.service.WechatServerService;
import com.caimei365.wechat.utils.MessageUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.util.Map;

/**
 * 微信公众号服务配置
 *
 * @author : Charles
 * @date : 2022/1/13
 */
@Slf4j
@RestController
public class WechatServerApi {

    @Resource
    private WechatServerService wechatServerService;

    /**
     * 验证消息的确来自微信服务器
     */
    @GetMapping("")
    public String validateSignature(HttpServletRequest request) throws UnsupportedEncodingException {
        request.setCharacterEncoding("UTF-8");
        return wechatServerService.validateSignature(request);
    }

    /**
     * 处理微信服务器的消息转发
     */
    @PostMapping("")
    public String handleMessage(HttpServletRequest request) throws UnsupportedEncodingException {
        request.setCharacterEncoding("UTF-8");
        long startProcessTime = System.currentTimeMillis();
        // xml格式的消息数据
        String respXml = null;
        try {
            // 调用parseXml方法解析请求消息
            Map<String, String> requestMap = MessageUtil.parseXml(request);
            // 消息类型
            String msgType = requestMap.get(WeChatConstant.MSG_TYPE);
            // 消息类型：事件推送
            if (msgType.equals(WeChatConstant.MESSAGE_TYPE_EVENT)) {
                // 事件类型
                String eventType = requestMap.get(WeChatConstant.EVENT);
                // subscribe(订阅/关注)
                if (eventType.equals(WeChatConstant.EVENT_TYPE_SUBSCRIBE)) {
                    respXml = wechatServerService.handleSubscribeMsg(requestMap);
                }
                // unsubscribe(取消订阅)，取消订阅后用户不会再收到公众账号发送的消息，因此回复"success"
                else if (eventType.equals(WeChatConstant.EVENT_TYPE_UNSUBSCRIBE)) {
                    respXml = "success";
                }
                // 扫描带参数二维码
                else if (eventType.equals(WeChatConstant.EVENT_TYPE_SCAN)) {
                    respXml = wechatServerService.handleScanMsg(requestMap);
                }
                // 上报地理位置
                else if (eventType.equals(WeChatConstant.EVENT_TYPE_LOCATION)) {
                    respXml = wechatServerService.handleLocationMsg(requestMap);
                }
                // 点击自定义菜单
                else if (eventType.equals(WeChatConstant.EVENT_TYPE_CLICK)) {
                    respXml = wechatServerService.handleClickMenuMsg(requestMap);
                }
            }
            // 图片消息
            else if (msgType.equals(WeChatConstant.MESSAGE_TYPE_IMAGE)) {
                // TODO 暂未开发
                respXml = "success";
            }
            // 语音消息
            else if (msgType.equals(WeChatConstant.MESSAGE_TYPE_VOICE)) {
                // TODO 暂未开发
                respXml = "success";
            }
            // 视频消息
            else if (msgType.equals(WeChatConstant.MESSAGE_TYPE_VIDEO)) {
                // TODO 暂未开发
                respXml = "success";
            }
            // 地理位置消息
            else if (msgType.equals(WeChatConstant.MESSAGE_TYPE_LOCATION)) {
                // TODO 暂未开发
                respXml = "success";
            }
            // 链接消息
            else if (msgType.equals(WeChatConstant.MESSAGE_TYPE_LINK)) {
                // TODO 暂未开发
                respXml = "success";
            }
            // 文本消息
            else if (msgType.equals(WeChatConstant.MESSAGE_TYPE_TEXT)) {
                respXml = wechatServerService.handleTextMsg(requestMap);
            }
            // 默认文本消息
            else {
                respXml = wechatServerService.handleTextMsg(requestMap);
            }
            if (respXml == null) { respXml = "success"; }
        } catch (Exception e) {
            log.info("微信公众号消息处理时出现异常：" + e.getMessage());
        }
        log.info(">>>>>>>>>>微信回复用户消息：" + respXml);
        long endProcessTime = System.currentTimeMillis();
        log.info(">>>>>>>>>>处理请求耗时:" + (endProcessTime - startProcessTime) + "ms");
        return respXml;
    }
}
