package com.caimei365.wechat.utils;

import com.caimei365.wechat.entity.WeChatConstant;
import com.caimei365.wechat.entity.WechatArticleDetail;
import lombok.extern.slf4j.Slf4j;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/1/13
 */
@Slf4j
public class MessageUtil {

    /**
     * 解析XML
     */
    public static Map<String, String> parseXml(HttpServletRequest request) throws Exception {
        // 将解析结果存储在HashMap中
        Map<String, String> map = new HashMap<>();
        // 从request中取得输入流
        InputStream inputStream = request.getInputStream();
        // 读取输入流
        SAXReader reader = new SAXReader();
        Document document = reader.read(inputStream);
        // 得到xml根元素
        Element root = document.getRootElement();
        // 得到根元素的所有子节点
        List<Element> elementList = root.elements();
        // 遍历所有子节点
        for (Element e : elementList) { map.put(e.getName(), e.getText()); }
        // 释放资源
        inputStream.close();
        return map;
    }

    /**
     * 包装xml根返回消息
     *  <xml>
     *   <ToUserName><![CDATA[toUser]]></ToUserName>
     *   <FromUserName><![CDATA[fromUser]]></FromUserName>
     *   <CreateTime>1399197672</CreateTime>
     *   <消息内容></消息内容>
     * </xml>
     * @return xml字符串
     */
    public static String setRootXml(String toUser, String fromUser, String xmlContent) {
        return "<xml>" +
                "<ToUserName><![CDATA[" + toUser + "]]></ToUserName>" +
                "<FromUserName><![CDATA[" + fromUser + "]]></FromUserName>" +
                "<CreateTime>" + System.currentTimeMillis() + "</CreateTime>" +
                xmlContent +
                "</xml>";
    }

    /**
     * 将消息转发到到客服系统
     *  <xml>
     *   <ToUserName><![CDATA[toUser]]></ToUserName>
     *   <FromUserName><![CDATA[fromUser]]></FromUserName>
     *   <CreateTime>1399197672</CreateTime>
     *   <MsgType><![CDATA[transfer_customer_service]]></MsgType>
     * </xml>
     */
    public static String setCustomerXml(String toUser, String fromUser) {
        String xmlContent = "<MsgType><![CDATA[transfer_customer_service]]></MsgType>";
        return setRootXml(toUser, fromUser, xmlContent);
    }

    /**
     * 包装xml文本内容
     * <xml>
     *   <ToUserName><![CDATA[toUser]]></ToUserName>
     *   <FromUserName><![CDATA[fromUser]]></FromUserName>
     *   <CreateTime>12345678</CreateTime>
     *   <MsgType><![CDATA[text]]></MsgType>
     *   <Content><![CDATA[你好]]></Content>
     * </xml>
     */
    public static String setTextXml(String toUser, String fromUser, String content) {
        String xmlContent = "<MsgType><![CDATA[" + WeChatConstant.MESSAGE_TYPE_TEXT + "]]></MsgType>"+
                "<Content><![CDATA[" + content + "]]></Content>";
        return setRootXml(toUser, fromUser, xmlContent);
    }

    /**
     * 包装xml图文内容
     * <xml>
     *   <ToUserName><![CDATA[toUser]]></ToUserName>
     *   <FromUserName><![CDATA[fromUser]]></FromUserName>
     *   <CreateTime>12345678</CreateTime>
     *   <MsgType><![CDATA[news]]></MsgType>
     *   <ArticleCount>1</ArticleCount>
     *   <Articles>
     *     <item>
     *       <Title><![CDATA[title1]]></Title>
     *       <Description><![CDATA[description1]]></Description>
     *       <PicUrl><![CDATA[picurl]]></PicUrl>
     *       <Url><![CDATA[url]]></Url>
     *     </item>
     *   </Articles>
     * </xml>
     */
    public static String setArticleXml(String toUser, String fromUser, List<WechatArticleDetail> articleList) {
        StringBuilder content = new StringBuilder();
        content.append("<MsgType><![CDATA[").append(WeChatConstant.MESSAGE_TYPE_NEWS).append("]]></MsgType>");
        content.append("<ArticleCount><![CDATA[").append(articleList.size()).append("]]></ArticleCount>");
        content.append("<Articles>");
        for (WechatArticleDetail article : articleList) {
            if (null != article) {
                content.append("<item>");
                content.append("<Title><![CDATA[").append(article.getTitle()).append("]]></Title>");
                content.append("<Description><![CDATA[").append(article.getDescription()).append("]]></Description>");
                content.append("<Url><![CDATA[").append(article.getLinkUrl()).append("]]></Url>");
                content.append("<PicUrl><![CDATA[").append(article.getPicUrl()).append("]]></PicUrl>");
                content.append("</item>");
            }
        }
        content.append("</Articles>");
        return setRootXml(toUser, fromUser, content.toString());
    }














//    /**
//     * (多层)map转换为xml格式字符串
//     *
//     * @param map 需要转换为xml的map
//     * @param isCDATA 是否加入CDATA标识符 true:加入 false:不加入
//     * @return xml字符串
//     */
//    public static String multiMapToXml(Map<String, Object> map, boolean isCDATA){
//        String parentName = "xml";
//        Document doc = DocumentHelper.createDocument();
//        doc.addElement(parentName);
//        return recursionMapToXml(doc.getRootElement(), parentName, map, isCDATA);
//    }
//
//    /**
//     * multilayerMapToXml核心方法，递归调用
//     *
//     * @param element 节点元素
//     * @param parentName 根元素属性名
//     * @param map 需要转换为xml的map
//     * @param isCDATA 是否加入CDATA标识符 true:加入 false:不加入
//     * @return xml字符串
//     */
//    private static String recursionMapToXml(Element element, String parentName, Map<String, Object> map, boolean isCDATA) {
//        Element xmlElement = element.addElement(parentName);
//        map.keySet().forEach(key -> {
//            Object obj = map.get(key);
//            if (obj instanceof Map) {
//                recursionMapToXml(xmlElement, key, (Map<String, Object>)obj, isCDATA);
//            } else {
//                String value = obj == null ? "" : obj.toString();
//                if (isCDATA) {
//                    xmlElement.addElement(key).addCDATA(value);
//                } else {
//                    xmlElement.addElement(key).addText(value);
//                }
//            }
//        });
//        return xmlElement.asXML();
//    }


//    /**
//     * 格式化xml,显示为容易看的XML格式
//     *
//     * @param xml 需要格式化的xml字符串
//     * @return
//     */
//    public static String formatXML(String xml) {
//        String requestXML = null;
//        try {
//            // 拿取解析器
//            SAXReader reader = new SAXReader();
//            Document document = reader.read(new StringReader(xml));
//            if (null != document) {
//                StringWriter stringWriter = new StringWriter();
//                // 格式化,每一级前的空格
//                OutputFormat format = new OutputFormat("    ", true);
//                // xml声明与内容是否添加空行
//                format.setNewLineAfterDeclaration(false);
//                // 是否设置xml声明头部
//                format.setSuppressDeclaration(false);
//                // 是否分行
//                format.setNewlines(true);
//                XMLWriter writer = new XMLWriter(stringWriter, format);
//                writer.write(document);
//                writer.flush();
//                writer.close();
//                requestXML = stringWriter.getBuffer().toString();
//            }
//            return requestXML;
//        } catch (Exception e) {
//            log.error("格式化xml，失败 --> {0}", e);
//            return null;
//        }
//    }
}
