package com.caimei365.wechat.utils;

import com.caimei365.wechat.entity.WeChatConstant;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/1/13
 */
public class SignUtil {

    /**
     * 加密/校验流程如下：
     *    1）将token、timestamp、nonce三个参数进行字典序排序
     *    2）将三个参数字符串拼接成一个字符串进行sha1加密
     *    3）开发者获得加密后的字符串可与signature对比，标识该请求来源于微信
     * @param signature 表示微信加密签名，signature结合了开发者填写的 token 参数和请求中的timestamp参数、nonce参数
     * @param timestamp 表示时间戳
     * @param nonce 表示随机数
     */
    public static boolean checkSignature(String signature, String timestamp, String nonce) {
        // 微信 - 服务器配置 -令牌(Token)
        String token = WeChatConstant.TOKEN;
        String[] arr = new String[] {token, timestamp, nonce};
        // 将token、timestamp、nonce三个参数进行字典序排序
        Arrays.sort(arr);
        StringBuilder content = new StringBuilder();
        for (String s : arr) {
            content.append(s);
        }
        MessageDigest md = null;
        String tmpStr = null;
        try {
            md = MessageDigest.getInstance("SHA-1");
            // 将三个参数字符串拼接成一个字符串进行sha1加密
            byte[] digest = md.digest(content.toString().getBytes());
            tmpStr = byteToStr(digest);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        // 将sha1加密后的字符串可与signature对比，标识该请求来源于微信
        return tmpStr != null && tmpStr.equals(signature.toUpperCase());
    }

    /**
     * 将字节数组转换为十六进制字符串
     */
    private static String byteToStr(byte[] byteArray) {
        StringBuilder strDigest = new StringBuilder();
        for (byte b : byteArray) {
            strDigest.append(byteToHexStr(b));
        }
        return strDigest.toString();
    }

    /**
     * 将字节转换为十六进制字符串
     */
    private static String byteToHexStr(byte mByte) {
        char[] digit = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        char[] tempArr = new char[2];
        tempArr[0] = digit[(mByte >>> 4) & 0X0F];
        tempArr[1] = digit[mByte & 0X0F];
        return new String(tempArr);
    }
}
